#import <Foundation/Foundation.h>
#import "SSCrypto.h"

int main (int argc, const char * argv[]) {
    
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
    int n = 0;
    NSData *seedData = [[SSCrypto getKeyDataWithLength:32] retain];
//    NSData *foo = [NSData dataWithBytes:"hello\0" length:6];
    SSCrypto *crypto = [[SSCrypto alloc] initWithSymmetricKey:seedData];
    NSArray *ciphers = [[NSArray alloc] initWithObjects:@"aes256", @"aes128",
        @"blowfish", @"aes192", @"RC4", @"blowfish", @"RC5", @"des3", @"des", nil];
    NSArray *digests = [[NSArray alloc] initWithObjects:@"MD2", @"MD4",
        @"MD5", @"SHA1", @"RIPEMD160", nil];
    void *hello = "hello\0";
	NSString *world = @"world";
	
    [crypto setClearTextWithData:[NSData dataWithBytes:hello length:6]];

    for (n = 0; n < [ciphers count]; n++){
        NSData *cipherText = [crypto encrypt:[ciphers objectAtIndex:n]];
        NSData *base64 = [SSCrypto encodeBase64Data:cipherText];
        NSData *clearText = [crypto decrypt:[ciphers objectAtIndex:n]];
        NSLog(@"clearText: %s", [clearText bytes]);
        [crypto setCipherText:cipherText];
        if (cipherText != nil && clearText != nil){
            NSLog(@"clear text: '%s' using %@",
                  [[crypto clearTextAsData] bytes], [ciphers objectAtIndex:n]);
            NSLog(@"cipher text: '%@' using %@",
                  cipherText, [ciphers objectAtIndex:n]);
            NSLog(@"base64 cipher text: '%s' using %@",
                  [base64 bytes], [ciphers objectAtIndex:n]);
            NSLog(@"base64 decoded cipher text: '%@' using %@",
                  [SSCrypto decodeBase64Data:base64], [ciphers objectAtIndex:n]);
            NSLog(@"decrypted text: '%s' using %@",
                  [clearText bytes], [ciphers objectAtIndex:n]);
			NSLog(@" ");
        }
    }
    
    [crypto setClearTextWithString:world];
    
    for (n = 0; n < [ciphers count]; n++){
        NSData *cipherText = [crypto encrypt:[ciphers objectAtIndex:n]];
        NSData *clearText = [crypto decrypt:[ciphers objectAtIndex:n]];
        [crypto setCipherText:cipherText];
        if (cipherText != nil && clearText != nil){
            NSLog(@"clear text: '%@' using %@",
                  [crypto clearTextAsString], [ciphers objectAtIndex:n]);
            NSLog(@"cipher text: '%@' using %@",
                  cipherText, [ciphers objectAtIndex:n]);
            NSLog(@"decrypted text: '%s' using %@",
                  [clearText bytes], [ciphers objectAtIndex:n]);
			NSLog(@" ");
        }
    }
    
    for (n = 0; n < [digests count]; n++)
    {
        NSLog(@"%@ digest hexdump: %@",
              [digests objectAtIndex:n],
              [[crypto digest:[digests objectAtIndex:n]] hexdump]);
    }
	
    NSString *sixty_four = @"this was base sixty four encoded.";
    NSData *enc64 = [SSCrypto encodeBase64Data:[sixty_four dataUsingEncoding:NSUTF8StringEncoding]];
    NSLog(@"base 64 enc: %@", enc64);
    NSLog(@"base 64 dec: %@", [NSString stringWithUTF8String:[[SSCrypto decodeBase64Data:enc64] bytes]]);
    
	// Entering asymmetric (just RSA, no DSA or DH or anything else) mode
	[crypto release];

	// Slurp the private and public keys from the files in this project
	NSString *tempPrivate = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Privatekey" ofType:@"pem" inDirectory:@"../../"]];
	NSString *tempPublic = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"PUBKEY" ofType:@"pem" inDirectory:@"../../"]];
    
	crypto = [[SSCrypto alloc] initWithRSAKeys:[tempPublic dataUsingEncoding:NSUTF8StringEncoding] private:[tempPrivate dataUsingEncoding:NSUTF8StringEncoding]];
    	
	//Now we go through the hello, world cycle like in the cipher loops above
	[crypto setClearTextWithData:[NSData dataWithBytes:hello length:6]];
	// don't use symmetric ciphers
	[crypto setSymmetric:NO];
	
	NSData *cipherText = [crypto encrypt];
	NSData *clearText = [crypto decrypt];
	[crypto setCipherText:cipherText];
	if (cipherText != nil && clearText != nil){
		NSLog(@"clear text: '%s' using RSA", [[crypto clearTextAsData] bytes]);
		NSLog(@"cipher text: '%@' using RSA", cipherText);
		NSLog(@"decrypted text: '%s' using RSA", [clearText bytes]);
		NSLog(@" ");
	}
	
    [crypto setClearTextWithString:world];
    
	cipherText = [crypto encrypt];
	clearText = [crypto decrypt];
	[crypto setCipherText:cipherText];
	if (cipherText != nil && clearText != nil){
		NSLog(@"clear text: '%@' using RSA", [crypto clearTextAsString]);
		NSLog(@"cipher text: '%@' using RSA", cipherText);
		NSLog(@"decrypted text: '%s' using RSA", [clearText bytes]);
		NSLog(@" ");
	}
	
	[digests release];
	[ciphers release];
    [crypto release];
	[seedData release];
    [pool release];
    return 0;
}
